from __future__ import print_function

import unittest
import sys
import threading

import wrapt
from wrapt.importer import _post_import_hooks

from compat import PY2, PY3

class TestPostImportHooks(unittest.TestCase):

    def setUp(self):
        super(TestPostImportHooks, self).setUp()

        # So we can import 'this' and test post-import hooks multiple times
        # below in the context of a single Python process, remove 'this' from
        # sys.modules and post import hooks.

        sys.modules.pop('this', None)
        _post_import_hooks.pop('this', None)

    def test_before_import(self):
        invoked = []

        @wrapt.when_imported('this')
        def hook_this(module):
            self.assertEqual(module.__name__, 'this')
            invoked.append(1)

        self.assertEqual(len(invoked), 0)

        import this

        self.assertEqual(len(invoked), 1)

    def test_after_import(self):
        invoked = []

        import this

        self.assertEqual(len(invoked), 0)

        @wrapt.when_imported('this')
        def hook_this(module):
            self.assertEqual(module.__name__, 'this')
            invoked.append(1)

        self.assertEqual(len(invoked), 1)

    def test_before_and_after_import(self):
        invoked_one = []
        invoked_two = []

        @wrapt.when_imported('this')
        def hook_this_one(module):
            self.assertEqual(module.__name__, 'this')
            invoked_one.append(1)

        self.assertEqual(len(invoked_one), 0)
        self.assertEqual(len(invoked_two), 0)

        import this

        self.assertEqual(len(invoked_one), 1)
        self.assertEqual(len(invoked_two), 0)

        @wrapt.when_imported('this')
        def hook_this_two(module):
            self.assertEqual(module.__name__, 'this')
            invoked_two.append(1)

        self.assertEqual(len(invoked_one), 1)
        self.assertEqual(len(invoked_two), 1)

    def test_remove_from_sys_modules(self):
        invoked = []

        @wrapt.when_imported('this')
        def hook_this(module):
            self.assertEqual(module.__name__, 'this')
            invoked.append(1)

        import this
        self.assertEqual(len(invoked), 1)

        del sys.modules['this']
        wrapt.register_post_import_hook(hook_this, 'this')
        import this

        self.assertEqual(len(invoked), 2)

    def test_import_deadlock_1(self):
        # This tries to verify that we haven't created a deadlock situation when
        # code executed from a post module import, for a module that has already
        # been imported, creates a thread which in turn attempts to register
        # another import hook.

        import this

        @wrapt.when_imported('this')
        def hook_this(module):
            def worker():
                @wrapt.when_imported('xxx')
                def hook_xxx(module):
                    pass

            thread = threading.Thread(target=worker)
            thread.start()
            thread.join(timeout=10)

            self.assertFalse(thread.is_alive())

        del sys.modules['this']

    def test_import_deadlock_2(self):
        # This tries to verify that we haven't created a deadlock situation when
        # code executed from a post module import, for a module that has not yet
        # been imported, creates a thread which in turn attempts to register
        # another import hook.

        @wrapt.when_imported('this')
        def hook_this(module):
            def worker():
                @wrapt.when_imported('xxx')
                def hook_xxx(module):
                    pass

            thread = threading.Thread(target=worker)
            thread.start()
            thread.join(timeout=10)

            self.assertFalse(thread.is_alive())

        import this
        del sys.modules['this']

    def test_import_deadlock_3(self):
        # This tries to verify that we haven't created a deadlock situation when
        # code executed from a post module import hook imports another module.

        # Note that we cannot run this test on Python 2.X as it has a single
        # global module import lock which means that if a thread runs during
        # module import and it in turns does an import that it will then block
        # on the parent thread which holds the global module import lock. This
        # is a fundamental behaviour of Python and not wrapt. In Python 3.X
        # there is a module import lock per named module and so we do not have
        # this problem.

        if PY2:
          return

        hooks_called = []

        @wrapt.when_imported('this')
        def hook_this(module):
            hooks_called.append('this')

            self.assertFalse('wsgiref' in sys.modules)
    
            @wrapt.when_imported('wsgiref')
            def hook_wsgiref(module):
                hooks_called.append('wsgiref')

            def worker():
                import wsgiref

            thread = threading.Thread(target=worker)
            thread.start()
            thread.join(timeout=10)

            self.assertFalse(thread.is_alive())

        import this
        del sys.modules['this']

        self.assertEqual(hooks_called, ['this', 'wsgiref'])

    def test_loader(self):
        @wrapt.when_imported('this')
        def hook_this(module):
            pass

        import this

        if sys.version_info[:2] >= (3, 3):
            from importlib.machinery import SourceFileLoader
            self.assertIsInstance(this.__loader__, SourceFileLoader)
            self.assertIsInstance(this.__spec__.loader, SourceFileLoader)

        else:
            self.assertEqual(hasattr(this, "__loader__"), False)

if __name__ == '__main__':
    unittest.main()



import sys

PY2 = sys.version_info[0] < 3
PY3 = sys.version_info[0] >= 3

PYXY = tuple(sys.version_info[:2])

if PY3:
    import builtins
    exec_ = getattr(builtins, "exec")
    del builtins

else:
    def exec_(_code_, _globs_=None, _locs_=None):
        """Execute code in a namespace."""
        if _globs_ is None:
            frame = sys._getframe(1)
            _globs_ = frame.f_globals
            if _locs_ is None:
                _locs_ = frame.f_locals
            del frame
        elif _locs_ is None:
            _locs_ = _globs_
        exec("""exec _code_ in _globs_, _locs_""")

    exec_("""def reraise(tp, value, tb=None):
    raise tp, value, tb
""")

try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec



from __future__ import print_function

import unittest

import wrapt

class TestDecorator(unittest.TestCase):

    def test_no_parameters(self):
        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        @wrapt.decorator
        def _decorator(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        @_decorator
        def _function(*args, **kwargs):
            return args, kwargs

        result = _function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

    def test_instance_method_as_decorator(self):
        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        class Instance(object):
            def __init__(self):
                self.count = 0
            @wrapt.decorator
            def decorator(self, wrapped, instance, args, kwargs):
                self.count += 1
                return wrapped(*args, **kwargs)

        instance = Instance()

        @instance.decorator
        def _function(*args, **kwargs):
            return args, kwargs

        result = _function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(instance.count, 1)

        result = _function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(instance.count, 2)

    def test_class_method_as_decorator(self):
        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        class Instance(object):
            count = 0
            @wrapt.decorator
            @classmethod
            def decorator(cls, wrapped, instance, args, kwargs):
                cls.count += 1
                return wrapped(*args, **kwargs)

        @Instance.decorator
        def _function(*args, **kwargs):
            return args, kwargs

        result = _function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(Instance.count, 1)

        result = _function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(Instance.count, 2)

    def test_class_type_as_decorator(self):
        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        @wrapt.decorator
        class ClassDecorator(object):
            def __call__(self, wrapped, instance, args, kwargs):
                return wrapped(*args, **kwargs)

        @ClassDecorator
        def _function(*args, **kwargs):
            return args, kwargs

        result = _function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

    def test_class_type_as_decorator_args(self):
        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        @wrapt.decorator
        class ClassDecorator(object):
            def __init__(self, arg):
                assert arg == 1
            def __call__(self, wrapped, instance, args, kwargs):
                return wrapped(*args, **kwargs)

        @ClassDecorator(arg=1)
        def _function(*args, **kwargs):
            return args, kwargs

        result = _function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

if __name__ == '__main__':
    unittest.main()



import unittest
import sys
from inspect import getfullargspec

from wrapt import formatargspec

class TestFormatargspec35(unittest.TestCase):

    def assertFormatEqual(self, func, ref):
        formatted = formatargspec(*getfullargspec(func))
        self.assertEqual(formatted, ref)

    def test_formatargspec(self):
        def foo1(): pass
        self.assertFormatEqual(foo1, '()')

        def foo2(a, b='c'): pass
        self.assertFormatEqual(foo2, ("(a, b='c')"))

        def foo3(a, b, *args, **kwargs): pass
        self.assertFormatEqual(foo3, '(a, b, *args, **kwargs)')

        def foo4(a: int, b) -> list: pass
        if sys.version_info[:2] < (3, 7):
            formatted4 = '(a:int, b) -> list'
        else:
            formatted4 = '(a: int, b) -> list'
        self.assertFormatEqual(foo4, formatted4)

        # examples from https://www.python.org/dev/peps/pep-3102/
        def sortwords(*wordlist, case_sensitive=False): pass
        self.assertFormatEqual(sortwords, '(*wordlist, case_sensitive=False)')

        def compare(a, b, *, key=None): pass
        self.assertFormatEqual(compare, '(a, b, *, key=None)')



__version_info__ = ('1', '16', '0rc1')
__version__ = '.'.join(__version_info__)

from .wrappers import (ObjectProxy, CallableObjectProxy, FunctionWrapper,
        BoundFunctionWrapper, WeakFunctionProxy, PartialCallableObjectProxy,
        resolve_path, apply_patch, wrap_object, wrap_object_attribute,
        function_wrapper, wrap_function_wrapper, patch_function_wrapper,
        transient_function_wrapper)

from .decorators import (adapter_factory, AdapterFactory, decorator,
        synchronized)

from .importer import (register_post_import_hook, when_imported,
        notify_module_loaded, discover_post_import_hooks)

# Import of inspect.getcallargs() included for backward compatibility. An
# implementation of this was previously bundled and made available here for
# Python <2.7. Avoid using this in future.

from inspect import getcallargs

# Variant of inspect.formatargspec() included here for forward compatibility.
# This is being done because Python 3.11 dropped inspect.formatargspec() but
# code for handling signature changing decorators relied on it. Exposing the
# bundled implementation here in case any user of wrapt was also needing it.

from .arguments import formatargspec



from __future__ import print_function

import unittest
import types

import wrapt

from compat import exec_, getfullargspec

DECORATORS_CODE = """
import wrapt

@wrapt.decorator
def passthru_decorator(wrapped, instance, args, kwargs):
    return wrapped(*args, **kwargs)
"""

decorators = types.ModuleType('decorators')
exec_(DECORATORS_CODE, decorators.__dict__, decorators.__dict__)

class Class(object):
    @classmethod
    def function(self, arg):
        '''documentation'''
        return arg

Original = Class

class Class(object):
    @decorators.passthru_decorator
    @classmethod
    def function(self, arg):
        '''documentation'''
        return arg

class TestNamingInnerClassMethod(unittest.TestCase):

    def test_class_object_name(self):
        # Test preservation of instance method __name__ attribute.

        self.assertEqual(Class.function.__name__,
                Original.function.__name__)

    def test_instance_object_name(self):
        # Test preservation of instance method __name__ attribute.

        self.assertEqual(Class().function.__name__,
                Original().function.__name__)

    def test_class_object_qualname(self):
        # Test preservation of instance method __qualname__ attribute.

        try:
            __qualname__ = Original.original.__qualname__
        except AttributeError:
            pass
        else:
            self.assertEqual(Class.function.__qualname__, __qualname__)

    def test_instance_object_qualname(self):
        # Test preservation of instance method __qualname__ attribute.

        try:
            __qualname__ = Original().original.__qualname__
        except AttributeError:
            pass
        else:
            self.assertEqual(Class().function.__qualname__, __qualname__)

    def test_class_module_name(self):
        # Test preservation of instance method __module__ attribute.

        self.assertEqual(Class.function.__module__,
                Original.function.__module__)

    def test_instance_module_name(self):
        # Test preservation of instance method __module__ attribute.

        self.assertEqual(Class().function.__module__,
                Original().function.__module__)

    def test_class_doc_string(self):
        # Test preservation of instance method __doc__ attribute.

        self.assertEqual(Class.function.__doc__,
                Original.function.__doc__)

    def test_instance_doc_string(self):
        # Test preservation of instance method __doc__ attribute.

        self.assertEqual(Class().function.__doc__,
                Original().function.__doc__)

    def test_class_argspec(self):
        # Test preservation of instance method argument specification.

        original_argspec = getfullargspec(Original.function)
        function_argspec = getfullargspec(Class.function)
        self.assertEqual(original_argspec, function_argspec)

    def test_instance_argspec(self):
        # Test preservation of instance method argument specification.

        original_argspec = getfullargspec(Original().function)
        function_argspec = getfullargspec(Class().function)
        self.assertEqual(original_argspec, function_argspec)

    def test_class_isinstance(self):
        # Test preservation of isinstance() checks.

        self.assertTrue(isinstance(Class.function,
                type(Original.function)))

    def test_instance_isinstance(self):
        # Test preservation of isinstance() checks.

        self.assertTrue(isinstance(Class().function,
                type(Original().function)))

class TestCallingInnerClassMethod(unittest.TestCase):

    def test_class_call_function(self):
        # Test calling classmethod.

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        @wrapt.decorator
        def _decorator(wrapped, instance, args, kwargs):
            self.assertEqual(instance, Class)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        @_decorator
        def _function(*args, **kwargs):
            return args, kwargs

        class Class(object):
            @_decorator
            @classmethod
            def _function(cls, *args, **kwargs):
                return (args, kwargs)

        result = Class._function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

    def test_instance_call_function(self):
        # Test calling classmethod via class instance.

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        @wrapt.decorator
        def _decorator(wrapped, instance, args, kwargs):
            self.assertEqual(instance, Class)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        @_decorator
        def _function(*args, **kwargs):
            return args, kwargs

        class Class(object):
            @_decorator
            @classmethod
            def _function(cls, *args, **kwargs):
                return (args, kwargs)

        result = Class()._function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

    def test_class_call_function_nested_decorators(self):
        # Test calling classmethod.

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        @wrapt.decorator
        def _decorator(wrapped, instance, args, kwargs):
            self.assertEqual(instance, Class)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        @_decorator
        def _function(*args, **kwargs):
            return args, kwargs

        class Class(object):
            @_decorator
            @_decorator
            @classmethod
            def _function(cls, *args, **kwargs):
                return (args, kwargs)

        result = Class._function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

    def test_instance_call_function_nested_decorators(self):
        # Test calling classmethod via class instance.

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        @wrapt.decorator
        def _decorator(wrapped, instance, args, kwargs):
            self.assertEqual(instance, Class)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        @_decorator
        def _function(*args, **kwargs):
            return args, kwargs

        class Class(object):
            @_decorator
            @_decorator
            @classmethod
            def _function(cls, *args, **kwargs):
                return (args, kwargs)

        result = Class()._function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

    def test_class_externally_applied_wrapper(self):
        # Test calling staticmethod via class when
        # the decorator has been applied from external to
        # the class using wrapping function.

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        class Class(object):
            @classmethod
            def _function(cls, *args, **kwargs):
                return (args, kwargs)

        def _decorator(wrapped, instance, args, kwargs):
            self.assertEqual(instance, Class)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        wrapt.wrap_function_wrapper(Class, "_function", _decorator)

        result = Class._function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

    def test_instance_externally_applied_wrapper(self):
        # Test calling staticmethod via class instance when
        # the decorator has been applied from external to
        # the class using wrapping function.

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        class Class(object):
            @classmethod
            def _function(cls, *args, **kwargs):
                return (args, kwargs)

        def _decorator(wrapped, instance, args, kwargs):
            self.assertEqual(instance, Class)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        wrapt.wrap_function_wrapper(Class, "_function", _decorator)

        result = Class()._function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

    def test_class_externally_passed_wrapper(self):
        # Test calling classmethod via class when
        # the decorator has been applied around reference
        # to the function.

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        class Class(object):
            @classmethod
            def _function(cls, *args, **kwargs):
                return (args, kwargs)

        @wrapt.decorator
        def _decorator(wrapped, instance, args, kwargs):
            self.assertEqual(instance, Class)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        _function = _decorator(Class._function)

        result = _function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

    def test_instance_externally_passed_wrapper(self):
        # Test calling classmethod via class instance when
        # the decorator has been applied around reference
        # to the function.

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        class Class(object):
            @classmethod
            def _function(cls, *args, **kwargs):
                return (args, kwargs)

        @wrapt.decorator
        def _decorator(wrapped, instance, args, kwargs):
            self.assertEqual(instance, Class)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        _function = _decorator(Class()._function)

        result = _function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

if __name__ == '__main__':
    unittest.main()



import sys

try:
    from pytest import File as FileCollector
except ImportError:
    from pytest.collect import File as FileCollector

version = tuple(sys.version_info[:2])

class DummyCollector(FileCollector):
    def collect(self):
        return []

def construct_dummy(path, parent):
    if hasattr(DummyCollector, "from_parent"):
        item = DummyCollector.from_parent(parent, fspath=path)
        return item
    else:
        return DummyCollector(path, parent=parent)

def pytest_pycollect_makemodule(path, parent):
    if '_py33' in path.basename and version < (3, 3):
        return construct_dummy(path, parent)
    if '_py34' in path.basename and version < (3, 4):
        return construct_dummy(path, parent)
    if '_py35' in path.basename and version < (3, 5):
        return construct_dummy(path, parent)
    if '_py36' in path.basename and version < (3, 6):
        return construct_dummy(path, parent)
    if '_py37' in path.basename and version < (3, 7):
        return construct_dummy(path, parent)
    if '_py38' in path.basename and version < (3, 8):
        return construct_dummy(path, parent)
    if '_py39' in path.basename and version < (3, 9):
        return construct_dummy(path, parent)
    if '_py310' in path.basename and version < (3, 10):
        return construct_dummy(path, parent)
    if '_py311' in path.basename and version < (3, 11):
        return construct_dummy(path, parent)
    if '_py312' in path.basename and version < (3, 12):
        return construct_dummy(path, parent)
    if '_py3' in path.basename and version < (3, 0):
        return construct_dummy(path, parent)
    if '_py2' in path.basename and version >= (3, 0):
        return construct_dummy(path, parent)



from __future__ import print_function

import unittest

import wrapt

from compat import PY2, PY3, exec_

@wrapt.decorator
def passthru_decorator(wrapped, instance, args, kwargs):
    return wrapped(*args, **kwargs)

class TestUpdateAttributes(unittest.TestCase):

    def test_update_name(self):
        @passthru_decorator
        def function():
            pass

        self.assertEqual(function.__name__, 'function')

        function.__name__ = 'override_name'

        self.assertEqual(function.__name__, 'override_name')

    def test_update_name_modified_on_original(self):
        def function():
            pass

        def wrapper(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        instance = wrapt.FunctionWrapper(function, wrapper)

        self.assertEqual(instance.__name__, 'function')

        instance.__name__ = 'override_name'

        self.assertEqual(function.__name__, 'override_name')
        self.assertEqual(instance.__name__, 'override_name')

    def test_update_qualname(self):

        @passthru_decorator
        def function():
            pass

        if PY3:
            method = self.test_update_qualname
            self.assertEqual(function.__qualname__,
                    (method.__qualname__ + '.<locals>.function'))

        function.__qualname__ = 'override_qualname'

        self.assertEqual(function.__qualname__, 'override_qualname')

    def test_update_qualname_modified_on_original(self):
        def function():
            pass

        def wrapper(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        instance = wrapt.FunctionWrapper(function, wrapper)

        if PY3:
            method = self.test_update_qualname_modified_on_original
            self.assertEqual(instance.__qualname__,
                    (method.__qualname__ + '.<locals>.function'))

        instance.__qualname__ = 'override_qualname'

        self.assertEqual(function.__qualname__, 'override_qualname')
        self.assertEqual(instance.__qualname__, 'override_qualname')

    def test_update_module(self):
        @passthru_decorator
        def function():
            pass

        self.assertEqual(function.__module__, __name__)

        function.__module__ = 'override_module'

        self.assertEqual(function.__module__, 'override_module')

    def test_update_module_modified_on_original(self):
        def function():
            pass

        def wrapper(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        instance = wrapt.FunctionWrapper(function, wrapper)

        self.assertEqual(instance.__module__, __name__)

        instance.__module__ = 'override_module'

        self.assertEqual(function.__module__, 'override_module')
        self.assertEqual(instance.__module__, 'override_module')

    def test_update_doc(self):
        @passthru_decorator
        def function():
            """documentation"""
            pass

        self.assertEqual(function.__doc__, "documentation")

        function.__doc__ = 'override_doc'

        self.assertEqual(function.__doc__, 'override_doc')

    def test_update_doc_modified_on_original(self):
        def function():
            """documentation"""
            pass

        def wrapper(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        instance = wrapt.FunctionWrapper(function, wrapper)

        self.assertEqual(instance.__doc__, "documentation")

        instance.__doc__ = 'override_doc'

        self.assertEqual(function.__doc__, 'override_doc')
        self.assertEqual(instance.__doc__, 'override_doc')

    def test_update_annotations(self):
        @passthru_decorator
        def function():
            pass

        if PY3:
            self.assertEqual(function.__annotations__, {})

        else:
            def run(*args):
                function.__annotations__

            self.assertRaises(AttributeError, run, ())

        override_annotations = {'override_annotations': ''}
        function.__annotations__ = override_annotations

        self.assertEqual(function.__wrapped__.__annotations__, override_annotations)
        self.assertEqual(function.__annotations__, override_annotations)

    def test_update_annotations_modified_on_original(self):
        def function():
            pass

        def wrapper(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        instance = wrapt.FunctionWrapper(function, wrapper)

        if PY3:
            self.assertEqual(instance.__annotations__, {})

        else:
            def run(*args):
                instance.__annotations__

            self.assertRaises(AttributeError, run, ())

        override_annotations = {'override_annotations': ''}
        instance.__annotations__ = override_annotations

        self.assertEqual(function.__annotations__, override_annotations)
        self.assertEqual(instance.__annotations__, override_annotations)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

import unittest
import inspect
import types

import wrapt

from compat import PY2, PY3, exec_

DECORATORS_CODE = """
import wrapt

@wrapt.decorator
def passthru_decorator(wrapped, instance, args, kwargs):
    return wrapped(*args, **kwargs)
"""

decorators = types.ModuleType('decorators')
exec_(DECORATORS_CODE, decorators.__dict__, decorators.__dict__)

class class1(object):
    pass

class1o = class1

class1d = decorators.passthru_decorator(class1)

class TestIntrospection(unittest.TestCase):

    def test_getmembers(self):
        class1o_members = inspect.getmembers(class1o)
        class1d_members = inspect.getmembers(class1d)

class TestInheritance(unittest.TestCase):

    def test_single_inheritance(self):
        @wrapt.decorator
        def passthru(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        @passthru
        class BaseClass(object):
            def __init__(self):
                self.value = 1

        class DerivedClass(BaseClass.__wrapped__):
            def __init__(self):
                super(DerivedClass, self).__init__()
                self.value = 2

        base = BaseClass()

        self.assertEqual(type(base), BaseClass.__wrapped__)
        self.assertTrue(isinstance(base, BaseClass.__wrapped__))
        self.assertEqual(base.value, 1)

        self.assertEqual(type(base).__mro__, (BaseClass.__wrapped__, object))

        derived = DerivedClass()

        self.assertEqual(type(derived), DerivedClass)
        self.assertTrue(isinstance(derived, BaseClass.__wrapped__))
        self.assertTrue(isinstance(derived, DerivedClass))
        self.assertEqual(derived.value, 2)

        self.assertEqual(type(derived).__mro__, (DerivedClass, BaseClass.__wrapped__, object))



from __future__ import print_function

import unittest

import wrapt

class TestObjectDescriptors(unittest.TestCase):

    def test_set_name(self):
        @wrapt.decorator
        def _decorator(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        attribute_name = []

        class _descriptor_wrapper:
            def __init__(self, descriptor):
                self.__wrapped__ = descriptor

            def __set_name__(self, owner, name):
                attribute_name.append(name)

            def __get__(self, instance, owner=None):
                return self.__wrapped__.__get__(instance, owner)

        class Instance(object):
            @_decorator
            @_descriptor_wrapper
            def method(self):
                return True

        instance = Instance()

        self.assertEqual(attribute_name, ["method"])
        self.assertEqual(instance.method(), True)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

import inspect
import unittest
import types

from typing import Iterable

import wrapt

from compat import PY2, exec_

DECORATORS_CODE = """
import wrapt
from typing import Iterable

def prototype1(arg1, arg2, arg3=None, *args, **kwargs) -> Iterable: pass
@wrapt.decorator(adapter=prototype1)
def adapter1(wrapped, instance, args, kwargs):
    '''adapter documentation'''
    return wrapped(*args, **kwargs)

def prototype2(arg1, arg2, arg3=None, *args, **kwargs) -> int: pass
@wrapt.decorator(adapter=prototype2)
def adapter2(wrapped, instance, args, kwargs):
    '''adapter documentation'''
    return wrapped(*args, **kwargs)
"""

decorators = types.ModuleType('decorators')
exec_(DECORATORS_CODE, decorators.__dict__, decorators.__dict__)

def function1(arg1, arg2) -> Iterable:
    '''documentation'''
    return arg1, arg2

function1o = function1

@decorators.adapter1
def function1(arg1, arg2) -> Iterable:
    '''documentation'''
    return arg1, arg2

function1d = function1

def function2(arg1, arg2) -> Iterable:
    '''documentation'''
    return arg1, arg2

function2o = function2

@decorators.adapter1
def function2(arg1, arg2) -> Iterable:
    '''documentation'''
    return arg1, arg2

function2d = function2

class TestAdapterAttributesWithAnnotations(unittest.TestCase):

    def test_annotations(self):
        # Test preservation of function __annotations__ attribute.

        self.assertEqual(function1d.__annotations__, function1o.__annotations__)

class TestArgumentSpecificationWithAnnotations(unittest.TestCase):

    def test_argspec(self):
        # Test preservation of function argument specification. It
        # actually needs to match that of the adapter function the
        # prototype of which was supplied via the dummy function.

        def _adapter(arg1, arg2, arg3=None, *args, **kwargs) -> Iterable: pass

        function1a_argspec = inspect.getfullargspec(_adapter)
        function1d_argspec = inspect.getfullargspec(function1d)
        self.assertEqual(function1a_argspec, function1d_argspec)

        # Now bind the function to an instance. The argspec should
        # still match.

        bound_function1d = function1d.__get__(object(), object)
        bound_function1d_argspec = inspect.getfullargspec(bound_function1d)
        self.assertEqual(function1a_argspec, bound_function1d_argspec)

    def test_signature(self):
        # Test preservation of function argument specification. It
        # actually needs to match that of the adapter function the
        # prototype of which was supplied via the dummy function.

        if PY2:
            return

        def _adapter(arg1, arg2, arg3=None, *args, **kwargs) -> Iterable: pass

        function1a_signature = str(inspect.signature(_adapter))
        function1d_signature = str(inspect.signature(function1d))
        self.assertEqual(function1a_signature, function1d_signature)

    def test_isinstance(self):
        # Test preservation of isinstance() checks.

        self.assertTrue(isinstance(function1d, type(function1o)))

class TestDynamicAdapterWithAnnotations(unittest.TestCase):

    def test_dynamic_adapter_function(self):
        def _adapter1(arg1, arg2, arg3=None, *args, **kwargs) -> Iterable: pass

        argspec1 = inspect.getfullargspec(_adapter1)

        @wrapt.decorator(adapter=argspec1)
        def _wrapper_1(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        @_wrapper_1
        def _function_1():
            pass

        self.assertEqual(inspect.getfullargspec(_function_1), argspec1)

        # Can't use a function signature with adapter factory which has
        # annotations which reference a non builtin type, so use test
        # function which returns int rather than Iterable.

        def _adapter2(arg1, arg2, arg3=None, *args, **kwargs) -> int: pass

        argspec2 = inspect.getfullargspec(_adapter2)

        args = '(arg1, arg2, arg3=None, *args, **kwargs) -> int'

        @wrapt.decorator(adapter=args)
        def _wrapper_2(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        @_wrapper_2
        def _function_2():
           pass

        self.assertEqual(inspect.getfullargspec(_function_2), argspec2)

    def test_dynamic_adapter_instancemethod(self):
        def _adapter1(self, arg1, arg2, arg3=None, *args, **kwargs) -> Iterable: pass

        argspec1 = inspect.getfullargspec(_adapter1)

        @wrapt.decorator(adapter=argspec1)
        def _wrapper_1(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        class Class1(object):
            @_wrapper_1
            def function(self):
                pass

        instance1 = Class1()

        self.assertEqual(inspect.getfullargspec(Class1.function), argspec1)
        self.assertEqual(inspect.getfullargspec(instance1.function), argspec1)

        # Can't use a function signature with adapter factory which has
        # annotations which reference a non builtin type.

        def _adapter2(self, arg1, arg2, arg3=None, *args, **kwargs) -> int: pass

        argspec2 = inspect.getfullargspec(_adapter2)

        args = '(self, arg1, arg2, arg3=None, *args, **kwargs) -> int'

        @wrapt.decorator(adapter=args)
        def _wrapper_2(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        class Class2(object):
            @_wrapper_2
            def function(self):
                pass

        instance2 = Class2()

        self.assertEqual(inspect.getfullargspec(Class2.function), argspec2)
        self.assertEqual(inspect.getfullargspec(instance2.function), argspec2)

    def test_dynamic_adapter_classmethod(self):
        def _adapter1(cls, arg1, arg2, arg3=None, *args, **kwargs) -> Iterable: pass

        argspec1 = inspect.getfullargspec(_adapter1)

        @wrapt.decorator(adapter=argspec1)
        def _wrapper_1(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        class Class1(object):
            @_wrapper_1
            @classmethod
            def function(cls):
                pass

        instance1 = Class1()

        self.assertEqual(inspect.getfullargspec(Class1.function), argspec1)
        self.assertEqual(inspect.getfullargspec(instance1.function), argspec1)

        # Can't use a function signature with adapter factory which has
        # annotations which reference a non builtin type.

        def _adapter2(cls, arg1, arg2, arg3=None, *args, **kwargs) -> int: pass

        argspec2 = inspect.getfullargspec(_adapter2)

        args = '(cls, arg1, arg2, arg3=None, *args, **kwargs) -> int'

        @wrapt.decorator(adapter=args)
        def _wrapper_2(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        class Class2(object):
            @_wrapper_2
            @classmethod
            def function(self):
                pass

        instance2 = Class2()

        self.assertEqual(inspect.getfullargspec(Class2.function), argspec2)
        self.assertEqual(inspect.getfullargspec(instance2.function), argspec2)

    def test_adapter_factory(self):
        def factory(wrapped):
            argspec = inspect.getfullargspec(wrapped)
            argspec.args.insert(0, 'arg0')
            return argspec

        @wrapt.decorator(adapter=wrapt.adapter_factory(factory))
        def _wrapper_1(wrapped, instance, args, kwargs):
            return wrapped(*args, **kwargs)

        @_wrapper_1
        def _function_1(arg1, arg2) -> Iterable:
            pass

        argspec = inspect.getfullargspec(_function_1)

        self.assertEqual(argspec.args, ['arg0', 'arg1', 'arg2'])
        self.assertEqual(argspec.annotations, {'return': Iterable})

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

import unittest

import wrapt

class TestArguments(unittest.TestCase):

    def test_getcallargs(self):
        def function(a, b=2, c=3, d=4, e=5, *args, **kwargs):
            pass

        expected = {'a': 10, 'c': 3, 'b': 20, 'e': 5, 'd': 40,
                'args': (), 'kwargs': {'f': 50}}
        calculated = wrapt.getcallargs(function, 10, 20, d=40, f=50)

        self.assertEqual(expected, calculated)

        expected = {'a': 10, 'c': 30, 'b': 20, 'e': 50, 'd': 40,
                'args': (60,), 'kwargs': {}}
        calculated = wrapt.getcallargs(function, 10, 20, 30, 40, 50, 60)

        self.assertEqual(expected, calculated)

    def test_unexpected_unicode_keyword(self):
        def function(a=2):
            pass

        kwargs = { u'b': 40 }
        self.assertRaises(TypeError, wrapt.getcallargs, function, **kwargs)



from __future__ import print_function

import unittest

import pickle

import wrapt

class CustomObjectProxy(wrapt.ObjectProxy):

    def __reduce_ex__(self, proto):
        return (list, (self.__wrapped__,))

class TestObjectPickle(unittest.TestCase):

    def test_pickle(self):
        proxy = wrapt.ObjectProxy([1])

        with self.assertRaises(NotImplementedError) as context:
            data = pickle.dumps(proxy)

        self.assertTrue(str(context.exception) ==
                'object proxy must define __reduce_ex__()')

    def test_pickle_proxy(self):
        proxy1 = CustomObjectProxy([1])
        pickled = pickle.dumps(proxy1)
        restored = pickle.loads(pickled)

        self.assertEqual(proxy1.__wrapped__, restored)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

import unittest
import sys

import wrapt

def global_function_1(*args, **kwargs):
    return args, kwargs

def global_function_2(*args, **kwargs):
    return args, kwargs

def global_function_3(*args, **kwargs):
    return args, kwargs

def global_function_3_enabled_literal_false(*args, **kwargs):
    return args, kwargs

def global_function_3_enabled_literal_true(*args, **kwargs):
    return args, kwargs

def global_function_3_enabled_callable(*args, **kwargs):
    return args, kwargs

def global_function_4(*args, **kwargs):
    return args, kwargs

class Class_1(object):
    def method(self, *args, **kwargs):
        return args, kwargs

class Class_2(object):
    @classmethod
    def method(cls, *args, **kwargs):
        return cls, args, kwargs

class Class_2_1(Class_2):
    pass

class Class_2_2(Class_2_1):
    pass

class Class_3(object):
    @staticmethod
    def method(*args, **kwargs):
        return args, kwargs

class TestMonkeyPatching(unittest.TestCase):

    def test_function_wrapper(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        @wrapt.function_wrapper
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        @wrapper
        def function(*args, **kwargs):
            return args, kwargs

        result = function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_function_wrapper_instance_method(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        _self = self

        class wrapper(object):
            @wrapt.function_wrapper
            def __call__(self, wrapped, instance, args, kwargs):
                _self.assertEqual(type(self), wrapper)
                called.append((args, kwargs))
                _self.assertEqual(instance, None)
                _self.assertEqual(args, _args)
                _self.assertEqual(kwargs, _kwargs)
                return wrapped(*args, **kwargs)

        @wrapper()
        def function(*args, **kwargs):
            return args, kwargs

        result = function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_function_wrapper_class_method(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        class wrapper(object):
            @wrapt.function_wrapper
            @classmethod
            def __call__(cls, wrapped, instance, args, kwargs):
                self.assertEqual(cls, wrapper)
                called.append((args, kwargs))
                self.assertEqual(instance, None)
                self.assertEqual(args, _args)
                self.assertEqual(kwargs, _kwargs)
                return wrapped(*args, **kwargs)

        @wrapper()
        def function(*args, **kwargs):
            return args, kwargs

        result = function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_wrap_function_module_name(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        wrapt.wrap_function_wrapper(__name__, 'global_function_1', wrapper)

        result = global_function_1(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_wrap_function_module(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        module = sys.modules[__name__]

        wrapt.wrap_function_wrapper(module, 'global_function_2', wrapper)

        result = global_function_2(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_wrap_instance_method_module_name(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        _instance = Class_1()

        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, _instance)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        wrapt.wrap_function_wrapper(__name__, 'Class_1.method',
                wrapper)

        result = _instance.method(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_wrap_class_method_module_name(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, Class_2)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        wrapt.wrap_function_wrapper(__name__, 'Class_2.method',
                wrapper)

        result = Class_2.method(*_args, **_kwargs)

        self.assertEqual(result, (Class_2, _args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_wrap_class_method_inherited(self):
        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        wrapt.wrap_function_wrapper(__name__, 'Class_2_1.method',
                wrapper)

        result = Class_2_1.method(*_args, **_kwargs)
        self.assertEqual(result, (Class_2_1, _args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

        called.pop()

        result = Class_2_2.method(*_args, **_kwargs)
        self.assertEqual(result, (Class_2_2, _args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_wrap_static_method_module_name(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        wrapt.wrap_function_wrapper(__name__, 'Class_3.method',
                wrapper)

        result = Class_3.method(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_patch_function_module_name(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        @wrapt.patch_function_wrapper(__name__, 'global_function_3')
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        result = global_function_3(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_patch_function_module_name_enabled_literal_false(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        @wrapt.patch_function_wrapper(__name__, 'global_function_3_enabled_literal_false', enabled=False)
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        result = global_function_3_enabled_literal_false(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called, [])

    def test_patch_function_module_name_enabled_literal_true(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        @wrapt.patch_function_wrapper(__name__, 'global_function_3_enabled_literal_true', enabled=True)
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        result = global_function_3_enabled_literal_true(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_patch_function_module_name_enabled_callable(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        enable = False

        def enabled():
            return enable

        @wrapt.patch_function_wrapper(__name__, 'global_function_3_enabled_callable', enabled=enabled)
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        result = global_function_3_enabled_callable(*_args, **_kwargs)

        enable = True

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called, [])

        result = global_function_3_enabled_callable(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_patch_function_module(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        module = sys.modules[__name__]

        @wrapt.patch_function_wrapper(module, 'global_function_4')
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        result = global_function_4(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def _test_transient_function_wrapper(self, *args, **kwargs):
        return args, kwargs

    def test_transient_function_wrapper(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        @wrapt.transient_function_wrapper(__name__,
                'TestMonkeyPatching._test_transient_function_wrapper')
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(wrapped, self._test_transient_function_wrapper)
            self.assertEqual(instance, self)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        @wrapper
        def function(*args, **kwargs):
            return self._test_transient_function_wrapper(*args, **kwargs)

        result = function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_transient_function_wrapper_instance_method(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        _self = self

        class wrapper(object):
            @wrapt.transient_function_wrapper(__name__,
                    'TestMonkeyPatching._test_transient_function_wrapper')
            def __call__(self, wrapped, instance, args, kwargs):
                called.append((args, kwargs))
                _self.assertEqual(wrapped, _self._test_transient_function_wrapper)
                _self.assertEqual(instance, _self)
                _self.assertEqual(args, _args)
                _self.assertEqual(kwargs, _kwargs)
                return wrapped(*args, **kwargs)

        @wrapper()
        def function(*args, **kwargs):
            return self._test_transient_function_wrapper(*args, **kwargs)

        result = function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

class TestExplicitMonkeyPatching(unittest.TestCase):

    def test_patch_instance_method_class(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        @wrapt.function_wrapper
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, _instance)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        class Class(object):
            def function(self, *args, **kwargs):
                return args, kwargs

        Class.function = wrapper(Class.function)

        _instance = Class()

        result = _instance.function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_patch_instance_method_dict(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        @wrapt.function_wrapper
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, _instance)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        class Class(object):
            def function(self, *args, **kwargs):
                return args, kwargs

        Class.function = wrapper(vars(Class)['function'])

        _instance = Class()

        result = _instance.function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_patch_instance_method_instance(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        @wrapt.function_wrapper
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, _instance)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        class Class(object):
            def function(self, *args, **kwargs):
                return args, kwargs

        _instance = Class()

        _instance.function = wrapper(_instance.function)

        result = _instance.function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

    def test_patch_instance_method_extracted(self):

        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        called = []

        @wrapt.function_wrapper
        def wrapper(wrapped, instance, args, kwargs):
            called.append((args, kwargs))
            self.assertEqual(instance, _instance)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        class Class(object):
            def function(self, *args, **kwargs):
                return args, kwargs

        _instance = Class()

        function = wrapper(_instance.function)

        result = function(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))
        self.assertEqual(called[0], (_args, _kwargs))

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

import unittest

import wrapt

class Class(object):
    def __init__(self, value):
        self.value = value

class TestAttributeProxy(unittest.TestCase):

    def test_wrap_attribute(self):
        wrapt.wrap_object_attribute(__name__, 'Class.value', wrapt.ObjectProxy)

        instance = Class(1)

        self.assertEqual(instance.value, 1)
        self.assertTrue(isinstance(instance.value, wrapt.ObjectProxy))

        instance.value = 2

        self.assertEqual(instance.value, 2)
        self.assertTrue(isinstance(instance.value, wrapt.ObjectProxy))



from __future__ import print_function

import unittest
import inspect
import types

import wrapt

from compat import PY2, PY3, exec_

DECORATORS_CODE = """
import wrapt

def prototype(arg1, arg2, *, arg3=None, **kwargs): pass
@wrapt.decorator(adapter=prototype)
def adapter1(wrapped, instance, args, kwargs):
    '''adapter documentation'''
    return wrapped(*args, **kwargs)
"""

decorators = types.ModuleType('decorators')
exec_(DECORATORS_CODE, decorators.__dict__, decorators.__dict__)

def function1(arg1, arg2):
    '''documentation'''
    return arg1, arg2

function1o = function1

@decorators.adapter1
def function1(arg1, arg2):
    '''documentation'''
    return arg1, arg2

function1d = function1

class TestArgumentSpecification(unittest.TestCase):

    def test_getfullargspec(self):
        # Test preservation of function argument specification. It
        # actually needs to match that of the adapter function the
        # prototype of which was supplied via the dummy function.

        def _adapter(arg1, arg2, *, arg3=None, **kwargs): pass

        function1a_argspec = inspect.getfullargspec(_adapter)
        function1d_argspec = inspect.getfullargspec(function1d)
        self.assertEqual(function1a_argspec, function1d_argspec)

    def test_signature(self):
        # Test preservation of function argument specification. It
        # actually needs to match that of the adapter function the
        # prototype of which was supplied via the dummy function.

        if PY2:
            return

        def _adapter(arg1, arg2, *, arg3=None, **kwargs): pass

        function1a_signature = str(inspect.signature(_adapter))
        function1d_signature = str(inspect.signature(function1d))
        self.assertEqual(function1a_signature, function1d_signature)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

import unittest

import copy

import wrapt

class CustomObjectProxy(wrapt.ObjectProxy):

    def __copy__(self):
        return CustomObjectProxy(copy.copy(self.__wrapped__))

    def __deepcopy__(self, memo):
        return CustomObjectProxy(copy.deepcopy(self.__wrapped__, memo))

class TestObjectCopy(unittest.TestCase):

    def test_copy(self):
        proxy1 = wrapt.ObjectProxy([1])

        with self.assertRaises(NotImplementedError) as context:
            proxy2 = copy.copy(proxy1)

        self.assertTrue(str(context.exception) ==
                'object proxy must define __copy__()')

    def test_deepcopy(self):
        proxy1 = wrapt.ObjectProxy([1])

        with self.assertRaises(NotImplementedError) as context:
            proxy2 = copy.deepcopy(proxy1)

        self.assertTrue(str(context.exception) ==
                'object proxy must define __deepcopy__()')

    def test_copy_proxy(self):
        proxy1 = CustomObjectProxy([1])
        proxy2 = copy.copy(proxy1)

        self.assertTrue(type(proxy1) == type(proxy2))
        self.assertEqual(proxy1, proxy2)
        self.assertEqual(proxy1.__wrapped__, proxy2.__wrapped__)

    def test_deepcopy_proxy(self):
        proxy1 = CustomObjectProxy([1])
        proxy2 = copy.deepcopy(proxy1)

        self.assertTrue(type(proxy1) == type(proxy2))
        self.assertEqual(proxy1, proxy2)
        self.assertEqual(proxy1.__wrapped__, proxy2.__wrapped__)

if __name__ == '__main__':
    unittest.main()



"""This module implements a post import hook mechanism styled after what is
described in PEP-369. Note that it doesn't cope with modules being reloaded.

"""

import sys
import threading

PY2 = sys.version_info[0] == 2

if PY2:
    string_types = basestring,
    find_spec = None
else:
    string_types = str,
    from importlib.util import find_spec

from .wrappers import ObjectProxy

# The dictionary registering any post import hooks to be triggered once
# the target module has been imported. Once a module has been imported
# and the hooks fired, the list of hooks recorded against the target
# module will be truncated but the list left in the dictionary. This
# acts as a flag to indicate that the module had already been imported.

_post_import_hooks = {}
_post_import_hooks_init = False
_post_import_hooks_lock = threading.RLock()

# Register a new post import hook for the target module name. This
# differs from the PEP-369 implementation in that it also allows the
# hook function to be specified as a string consisting of the name of
# the callback in the form 'module:function'. This will result in a
# proxy callback being registered which will defer loading of the
# specified module containing the callback function until required.

def _create_import_hook_from_string(name):
    def import_hook(module):
        module_name, function = name.split(':')
        attrs = function.split('.')
        __import__(module_name)
        callback = sys.modules[module_name]
        for attr in attrs:
            callback = getattr(callback, attr)
        return callback(module)
    return import_hook

def register_post_import_hook(hook, name):
    # Create a deferred import hook if hook is a string name rather than
    # a callable function.

    if isinstance(hook, string_types):
        hook = _create_import_hook_from_string(hook)

    with _post_import_hooks_lock:
        # Automatically install the import hook finder if it has not already
        # been installed.

        global _post_import_hooks_init

        if not _post_import_hooks_init:
            _post_import_hooks_init = True
            sys.meta_path.insert(0, ImportHookFinder())

        # Check if the module is already imported. If not, register the hook
        # to be called after import.

        module = sys.modules.get(name, None)

        if module is None:
            _post_import_hooks.setdefault(name, []).append(hook)

    # If the module is already imported, we fire the hook right away. Note that
    # the hook is called outside of the lock to avoid deadlocks if code run as a
    # consequence of calling the module import hook in turn triggers a separate
    # thread which tries to register an import hook.

    if module is not None:
        hook(module)

# Register post import hooks defined as package entry points.

def _create_import_hook_from_entrypoint(entrypoint):
    def import_hook(module):
        __import__(entrypoint.module_name)
        callback = sys.modules[entrypoint.module_name]
        for attr in entrypoint.attrs:
            callback = getattr(callback, attr)
        return callback(module)
    return import_hook

def discover_post_import_hooks(group):
    try:
        import pkg_resources
    except ImportError:
        return

    for entrypoint in pkg_resources.iter_entry_points(group=group):
        callback = _create_import_hook_from_entrypoint(entrypoint)
        register_post_import_hook(callback, entrypoint.name)

# Indicate that a module has been loaded. Any post import hooks which
# were registered against the target module will be invoked. If an
# exception is raised in any of the post import hooks, that will cause
# the import of the target module to fail.

def notify_module_loaded(module):
    name = getattr(module, '__name__', None)

    with _post_import_hooks_lock:
        hooks = _post_import_hooks.pop(name, ())

    # Note that the hook is called outside of the lock to avoid deadlocks if
    # code run as a consequence of calling the module import hook in turn
    # triggers a separate thread which tries to register an import hook.

    for hook in hooks:
        hook(module)

# A custom module import finder. This intercepts attempts to import
# modules and watches out for attempts to import target modules of
# interest. When a module of interest is imported, then any post import
# hooks which are registered will be invoked.

class _ImportHookLoader:

    def load_module(self, fullname):
        module = sys.modules[fullname]
        notify_module_loaded(module)

        return module

class _ImportHookChainedLoader(ObjectProxy):

    def __init__(self, loader):
        super(_ImportHookChainedLoader, self).__init__(loader)

        if hasattr(loader, "load_module"):
          self.__self_setattr__('load_module', self._self_load_module)
        if hasattr(loader, "create_module"):
          self.__self_setattr__('create_module', self._self_create_module)
        if hasattr(loader, "exec_module"):
          self.__self_setattr__('exec_module', self._self_exec_module)

    def _self_set_loader(self, module):
        # Set module's loader to self.__wrapped__ unless it's already set to
        # something else. Import machinery will set it to spec.loader if it is
        # None, so handle None as well. The module may not support attribute
        # assignment, in which case we simply skip it. Note that we also deal
        # with __loader__ not existing at all. This is to future proof things
        # due to proposal to remove the attribue as described in the GitHub
        # issue at https://github.com/python/cpython/issues/77458. Also prior
        # to Python 3.3, the __loader__ attribute was only set if a custom
        # module loader was used. It isn't clear whether the attribute still
        # existed in that case or was set to None.

        class UNDEFINED: pass

        if getattr(module, "__loader__", UNDEFINED) in (None, self):
            try:
                module.__loader__ = self.__wrapped__
            except AttributeError:
                pass

        if (getattr(module, "__spec__", None) is not None
                and getattr(module.__spec__, "loader", None) is self):
            module.__spec__.loader = self.__wrapped__

    def _self_load_module(self, fullname):
        module = self.__wrapped__.load_module(fullname)
        self._self_set_loader(module)
        notify_module_loaded(module)

        return module

    # Python 3.4 introduced create_module() and exec_module() instead of
    # load_module() alone. Splitting the two steps.

    def _self_create_module(self, spec):
        return self.__wrapped__.create_module(spec)

    def _self_exec_module(self, module):
        self._self_set_loader(module)
        self.__wrapped__.exec_module(module)
        notify_module_loaded(module)

class ImportHookFinder:

    def __init__(self):
        self.in_progress = {}

    def find_module(self, fullname, path=None):
        # If the module being imported is not one we have registered
        # post import hooks for, we can return immediately. We will
        # take no further part in the importing of this module.

        with _post_import_hooks_lock:
            if fullname not in _post_import_hooks:
                return None

        # When we are interested in a specific module, we will call back
        # into the import system a second time to defer to the import
        # finder that is supposed to handle the importing of the module.
        # We set an in progress flag for the target module so that on
        # the second time through we don't trigger another call back
        # into the import system and cause a infinite loop.

        if fullname in self.in_progress:
            return None

        self.in_progress[fullname] = True

        # Now call back into the import system again.

        try:
            if not find_spec:
                # For Python 2 we don't have much choice but to
                # call back in to __import__(). This will
                # actually cause the module to be imported. If no
                # module could be found then ImportError will be
                # raised. Otherwise we return a loader which
                # returns the already loaded module and invokes
                # the post import hooks.

                __import__(fullname)

                return _ImportHookLoader()

            else:
                # For Python 3 we need to use find_spec().loader
                # from the importlib.util module. It doesn't actually
                # import the target module and only finds the
                # loader. If a loader is found, we need to return
                # our own loader which will then in turn call the
                # real loader to import the module and invoke the
                # post import hooks.

                loader = getattr(find_spec(fullname), "loader", None)

                if loader and not isinstance(loader, _ImportHookChainedLoader):
                    return _ImportHookChainedLoader(loader)

        finally:
            del self.in_progress[fullname]

    def find_spec(self, fullname, path=None, target=None):
        # Since Python 3.4, you are meant to implement find_spec() method
        # instead of find_module() and since Python 3.10 you get deprecation
        # warnings if you don't define find_spec().

        # If the module being imported is not one we have registered
        # post import hooks for, we can return immediately. We will
        # take no further part in the importing of this module.

        with _post_import_hooks_lock:
            if fullname not in _post_import_hooks:
                return None

        # When we are interested in a specific module, we will call back
        # into the import system a second time to defer to the import
        # finder that is supposed to handle the importing of the module.
        # We set an in progress flag for the target module so that on
        # the second time through we don't trigger another call back
        # into the import system and cause a infinite loop.

        if fullname in self.in_progress:
            return None

        self.in_progress[fullname] = True

        # Now call back into the import system again.

        try:
            # This should only be Python 3 so find_spec() should always
            # exist so don't need to check.

            spec = find_spec(fullname)
            loader = getattr(spec, "loader", None)

            if loader and not isinstance(loader, _ImportHookChainedLoader):
                spec.loader = _ImportHookChainedLoader(loader)

            return spec

        finally:
            del self.in_progress[fullname]

# Decorator for marking that a function should be called as a post
# import hook when the target module is imported.

def when_imported(name):
    def register(hook):
        register_post_import_hook(hook, name)
        return hook
    return register



import os
import sys
import platform
import setuptools


# # --- Detect if extensions should be disabled ------------------------------

wrapt_env = os.environ.get('WRAPT_INSTALL_EXTENSIONS')

if wrapt_env is None:
    wrapt_env = os.environ.get('WRAPT_EXTENSIONS')

if wrapt_env is not None:
    disable_extensions = wrapt_env.lower() == 'false'
    force_extensions = wrapt_env.lower() == 'true'
else:
    if platform.system() == 'Windows' and sys.version_info[0] < 3:
      disable_extensions = True
      force_extensions = False
    else:
      disable_extensions = False
      force_extensions = False

if platform.python_implementation() != "CPython":
    disable_extensions = True

# --- C extension ------------------------------------------------------------

extensions = [
    setuptools.Extension(
        "wrapt._wrappers",
        sources=["src/wrapt/_wrappers.c"],
        optional=not force_extensions,
    )
]


# --- Setup ------------------------------------------------------------------

setuptools.setup(
    ext_modules=[] if disable_extensions else extensions
)



from __future__ import print_function

import unittest

import wrapt

from compat import PYXY

@wrapt.synchronized
def function():
    print('function')

class C1(object):

    @wrapt.synchronized
    def function1(self):
        print('function1')

    @wrapt.synchronized
    @classmethod
    def function2(cls):
        print('function2')

    @wrapt.synchronized
    @staticmethod
    def function3():
        print('function3')

c1 = C1()

@wrapt.synchronized
class C2(object):
    pass

@wrapt.synchronized
class C3:
    pass

class C4(object):

    # Prior to Python 3.9, this yields undesirable results due to how
    # class method is implemented. The classmethod doesn't bind the
    # method to the class before calling. As a consequence, the
    # decorator wrapper function sees the instance as None with the
    # class being explicitly passed as the first argument. It isn't
    # possible to detect and correct this. For more details see:
    # https://bugs.python.org/issue19072

    @classmethod
    @wrapt.synchronized
    def function2(cls):
        print('function2')

    @staticmethod
    @wrapt.synchronized
    def function3():
        print('function3')

c4 = C4()

class C5(object):

    def __bool__(self):
        return False
    __nonzero__=__bool__

    @wrapt.synchronized
    def function1(self):
        print('function1')

c5 = C5()

class TestSynchronized(unittest.TestCase):

    def test_synchronized_function(self):
        _lock0 = getattr(function, '_synchronized_lock', None)
        self.assertEqual(_lock0, None)

        function()

        _lock1 = getattr(function, '_synchronized_lock', None)
        self.assertNotEqual(_lock1, None)

        function()

        _lock2 = getattr(function, '_synchronized_lock', None)
        self.assertNotEqual(_lock2, None)
        self.assertEqual(_lock2, _lock1)

        function()

        _lock3 = getattr(function, '_synchronized_lock', None)
        self.assertNotEqual(_lock3, None)
        self.assertEqual(_lock3, _lock2)

    def test_synchronized_inner_staticmethod(self):
        _lock0 = getattr(C1.function3, '_synchronized_lock', None)
        self.assertEqual(_lock0, None)

        c1.function3()

        _lock1 = getattr(C1.function3, '_synchronized_lock', None)
        self.assertNotEqual(_lock1, None)

        C1.function3()

        _lock2 = getattr(C1.function3, '_synchronized_lock', None)
        self.assertNotEqual(_lock2, None)
        self.assertEqual(_lock2, _lock1)

        C1.function3()

        _lock3 = getattr(C1.function3, '_synchronized_lock', None)
        self.assertNotEqual(_lock3, None)
        self.assertEqual(_lock3, _lock2)

    def test_synchronized_outer_staticmethod(self):
        _lock0 = getattr(C4.function3, '_synchronized_lock', None)
        self.assertEqual(_lock0, None)

        c4.function3()

        _lock1 = getattr(C4.function3, '_synchronized_lock', None)
        self.assertNotEqual(_lock1, None)

        C4.function3()

        _lock2 = getattr(C4.function3, '_synchronized_lock', None)
        self.assertNotEqual(_lock2, None)
        self.assertEqual(_lock2, _lock1)

        C4.function3()

        _lock3 = getattr(C4.function3, '_synchronized_lock', None)
        self.assertNotEqual(_lock3, None)
        self.assertEqual(_lock3, _lock2)

    def test_synchronized_inner_classmethod(self):
        if hasattr(C1, '_synchronized_lock'):
            del C1._synchronized_lock

        _lock0 = getattr(C1, '_synchronized_lock', None)
        self.assertEqual(_lock0, None)

        c1.function2()

        _lock1 = getattr(C1, '_synchronized_lock', None)
        self.assertNotEqual(_lock1, None)

        C1.function2()

        _lock2 = getattr(C1, '_synchronized_lock', None)
        self.assertNotEqual(_lock2, None)
        self.assertEqual(_lock2, _lock1)

        C1.function2()

        _lock3 = getattr(C1, '_synchronized_lock', None)
        self.assertNotEqual(_lock3, None)
        self.assertEqual(_lock3, _lock2)

    def test_synchronized_outer_classmethod(self):
        # Prior to Python 3.9 this isn't detected as a class method
        # call, as the classmethod decorator doesn't bind the wrapped
        # function to the class before calling and just calls it direct,
        # explicitly passing the class as first argument. For more
        # details see: https://bugs.python.org/issue19072

        if PYXY < (3, 9):
            _lock0 = getattr(C4.function2, '_synchronized_lock', None)
        else:
            _lock0 = getattr(C4, '_synchronized_lock', None)
        self.assertEqual(_lock0, None)

        c4.function2()

        if PYXY < (3, 9):
            _lock1 = getattr(C4.function2, '_synchronized_lock', None)
        else:
            _lock1 = getattr(C4, '_synchronized_lock', None)
        self.assertNotEqual(_lock1, None)

        C4.function2()

        if PYXY < (3, 9):
            _lock2 = getattr(C4.function2, '_synchronized_lock', None)
        else:
            _lock2 = getattr(C4, '_synchronized_lock', None)
        self.assertNotEqual(_lock2, None)
        self.assertEqual(_lock2, _lock1)

        C4.function2()

        if PYXY < (3, 9):
            _lock3 = getattr(C4.function2, '_synchronized_lock', None)
        else:
            _lock3 = getattr(C4, '_synchronized_lock', None)
        self.assertNotEqual(_lock3, None)
        self.assertEqual(_lock3, _lock2)

    def test_synchronized_instancemethod(self):
        if hasattr(C1, '_synchronized_lock'):
            del C1._synchronized_lock

        _lock0 = getattr(c1, '_synchronized_lock', None)
        self.assertEqual(_lock0, None)

        C1.function1(c1)

        _lock1 = getattr(c1, '_synchronized_lock', None)
        self.assertNotEqual(_lock1, None)

        c1.function1()

        _lock2 = getattr(c1, '_synchronized_lock', None)
        self.assertNotEqual(_lock2, None)
        self.assertEqual(_lock2, _lock1)

        c1.function1()

        _lock3 = getattr(c1, '_synchronized_lock', None)
        self.assertNotEqual(_lock3, None)
        self.assertEqual(_lock3, _lock2)

        del c1._synchronized_lock

        C1.function2()

        _lock4 = getattr(C1, '_synchronized_lock', None)
        self.assertNotEqual(_lock4, None)

        c1.function1()

        _lock5 = getattr(c1, '_synchronized_lock', None)
        self.assertNotEqual(_lock5, None)
        self.assertNotEqual(_lock5, _lock4)

    def test_synchronized_type_new_style(self):
        if hasattr(C2, '_synchronized_lock'):
            del C2._synchronized_lock

        _lock0 = getattr(C2, '_synchronized_lock', None)
        self.assertEqual(_lock0, None)

        c2 = C2()

        _lock1 = getattr(C2, '_synchronized_lock', None)
        self.assertNotEqual(_lock1, None)

        c2 = C2()

        _lock2 = getattr(C2, '_synchronized_lock', None)
        self.assertNotEqual(_lock2, None)
        self.assertEqual(_lock2, _lock1)

        c2 = C2()

        _lock3 = getattr(C2, '_synchronized_lock', None)
        self.assertNotEqual(_lock3, None)
        self.assertEqual(_lock3, _lock2)

    def test_synchronized_type_old_style(self):
        if hasattr(C3, '_synchronized_lock'):
            del C3._synchronized_lock

        _lock0 = getattr(C3, '_synchronized_lock', None)
        self.assertEqual(_lock0, None)

        c2 = C3()

        _lock1 = getattr(C3, '_synchronized_lock', None)
        self.assertNotEqual(_lock1, None)

        c2 = C3()

        _lock2 = getattr(C3, '_synchronized_lock', None)
        self.assertNotEqual(_lock2, None)
        self.assertEqual(_lock2, _lock1)

        c2 = C3()

        _lock3 = getattr(C3, '_synchronized_lock', None)
        self.assertNotEqual(_lock3, None)
        self.assertEqual(_lock3, _lock2)

    def test_synchronized_false_instance(self):
        c5.function1()

        self.assertEqual(bool(c5), False)

        _lock1 = getattr(C5, '_synchronized_lock', None)
        self.assertEqual(_lock1, None)

        _lock2 = getattr(c5, '_synchronized_lock', None)
        self.assertNotEqual(_lock2, None)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

import unittest

import wrapt

class TestPartialCallableObjectProxy(unittest.TestCase):

    def test_no_arguments(self):
        def func0():
            return ((), {})

        partial0 = wrapt.PartialCallableObjectProxy(func0)

        args, kwargs = (), {}

        self.assertEqual(partial0(), (args, kwargs))

    def test_empty_arguments(self):
        def func0(*args, **kwargs):
            return (args, kwargs)

        args, kwargs = (), {}

        partial0 = wrapt.PartialCallableObjectProxy(func0, *args, **kwargs)

        self.assertEqual(partial0(), (args, kwargs))

    def test_1_positional_argument(self):
        def func0(*args, **kwargs):
            return (args, kwargs)

        args, kwargs = (1,), {}

        partial0 = wrapt.PartialCallableObjectProxy(func0, *args)

        self.assertEqual(partial0(), (args, kwargs))

    def test_1_keyword_argument(self):
        def func0(*args, **kwargs):
            return (args, kwargs)

        args, kwargs = (), {'k1': 1}

        partial0 = wrapt.PartialCallableObjectProxy(func0, **kwargs)

        self.assertEqual(partial0(), (args, kwargs))

    def test_multiple_positional_arguments(self):
        def func0(*args, **kwargs):
            return (args, kwargs)

        args, kwargs = (1, 2, 3), {}

        partial0 = wrapt.PartialCallableObjectProxy(func0, *args)

        self.assertEqual(partial0(), (args, kwargs))

    def test_multiple_keyword_arguments(self):
        def func0(*args, **kwargs):
            return (args, kwargs)

        args, kwargs = (), {'k1': 1, 'k2': 2, 'k3': 3}

        partial0 = wrapt.PartialCallableObjectProxy(func0, **kwargs)

        self.assertEqual(partial0(), (args, kwargs))

if __name__ == '__main__':
    unittest.main()



import unittest
import sys
from inspect import getfullargspec

from wrapt import formatargspec

class TestFormatargspec38(unittest.TestCase):

    def assertFormatEqual(self, func, ref):
        formatted = formatargspec(*getfullargspec(func))
        self.assertEqual(formatted, ref)

    def test_formatargspec(self):
        # exemples from https://www.python.org/dev/peps/pep-0570/
        def name1(p1, p2, /, p_or_kw, *, kw): pass
        self.assertFormatEqual(name1, '(p1, p2, p_or_kw, *, kw)')

        def name2(p1, p2=None, /, p_or_kw=None, *, kw): pass
        self.assertFormatEqual(name2, '(p1, p2=None, p_or_kw=None, *, kw)')

        def name3(p1, p2=None, /, *, kw): pass
        self.assertFormatEqual(name3, '(p1, p2=None, *, kw)')

        def name4(p1, p2=None, /): pass
        self.assertFormatEqual(name4, '(p1, p2=None)')

        def name5(p1, p2, /, p_or_kw): pass
        self.assertFormatEqual(name5, '(p1, p2, p_or_kw)')

        def name6(p1, p2, /): pass
        self.assertFormatEqual(name6, '(p1, p2)')

        def name7(p_or_kw, *, kw): pass
        self.assertFormatEqual(name7, '(p_or_kw, *, kw)')

        def name8(*, kw): pass
        self.assertFormatEqual(name8, '(*, kw)')



from __future__ import print_function

import unittest
import types

import wrapt

from compat import exec_, getfullargspec

DECORATORS_CODE = """
import wrapt

@wrapt.decorator
def passthru_decorator(wrapped, instance, args, kwargs):
    return wrapped(*args, **kwargs)
"""

decorators = types.ModuleType('decorators')
exec_(DECORATORS_CODE, decorators.__dict__, decorators.__dict__)

def function1():
    def inner(arg):
        '''documentation'''
        return arg
    return inner

function1o = function1

def function1():
    @decorators.passthru_decorator
    def inner(arg):
        '''documentation'''
        return arg
    return inner

function1d = function1

class TestNamingNestedFunction(unittest.TestCase):

    def test_object_name(self):
        # Test preservation of function __name__ attribute.

        self.assertEqual(function1d().__name__, function1o().__name__)

    def test_object_qualname(self):
        # Test preservation of function __qualname__ attribute.

        try:
            __qualname__ = function1o().__qualname__
        except AttributeError:
            pass
        else:
            self.assertEqual(function1d().__qualname__, __qualname__)

    def test_module_name(self):
        # Test preservation of function __module__ attribute.

        self.assertEqual(function1d().__module__, __name__)

    def test_doc_string(self):
        # Test preservation of function __doc__ attribute.

        self.assertEqual(function1d().__doc__, function1o().__doc__)

    def test_argspec(self):
        # Test preservation of function argument specification.

        function1o_argspec = getfullargspec(function1o())
        function1d_argspec = getfullargspec(function1d())
        self.assertEqual(function1o_argspec, function1d_argspec)

    def test_isinstance(self):
        # Test preservation of isinstance() checks.

        self.assertTrue(isinstance(function1d(), type(function1o())))

class TestCallingNestedFunction(unittest.TestCase):

    def test_call_function(self):
        _args = (1, 2)
        _kwargs = {'one': 1, 'two': 2}

        @wrapt.decorator
        def _decorator(wrapped, instance, args, kwargs):
            self.assertEqual(instance, None)
            self.assertEqual(args, _args)
            self.assertEqual(kwargs, _kwargs)
            return wrapped(*args, **kwargs)

        @_decorator
        def _function(*args, **kwargs):
            return args, kwargs

        def _function():
            @decorators.passthru_decorator
            def inner(*args, **kwargs):
                return args, kwargs
            return inner

        result = _function()(*_args, **_kwargs)

        self.assertEqual(result, (_args, _kwargs))

if __name__ == '__main__':
    unittest.main()
